//--------------------------------------------------------------//
//------------------------TeleportMaster------------------------//
//---------------------------EVILZIXIE--------------------------// 
//-----------------------------子邪-----------------------------//
//--------------------------------------------------------------//
const PLUGIN_NAME = "TeleportMaster";
const PLUGIN_DESC = "集成传送系统";
const PLUGIN_VERSION = [1, 0, 0];

ll.registerPlugin(PLUGIN_NAME, PLUGIN_DESC, PLUGIN_VERSION, {});

const BASE_DIR = `./plugins/${PLUGIN_NAME}/`;
const CONFIG_PATH = BASE_DIR + "config.json";
const HOME_DATA_PATH = BASE_DIR + "homes.json";
const WARP_DATA_PATH = BASE_DIR + "warps.json";
const SERVERS_PATH = BASE_DIR + "servers.json";
const DEATH_RECORDS_PATH = BASE_DIR + "death_records.json";

if (!File.exists(BASE_DIR)) {
    File.createDir(BASE_DIR);
}

const defaultConfig = {
    economyType: "llmoney",
    scoreboardObj: "money",
    
    modulesEnabled: {
        home: true,
        tpa: true,
        tpr: true,
        tpz: true,
        warp: true,
        server: true,
        deathtp: true    
    },
    
    home: {
        maxHomes: 3,
        baseSetCost: 100,
        incSetCost: 50,
        dimsEnabled: [true, true, true],
        dimsCost: [0, 0, 0],
        teleportCooldownSec: 60
    },
    
    tpa: {
        isDelayEnabled: true,
        maxDelay: 20,
        requestTimeout: 60,
        promptType: "form"
    },
    
    tpr: {
        dimsEnabled: [true, true, true],
        costPerDim: [0, 0, 0],
        maxRangePerDim: [3000, 1000, 1000],
        centerMethod: "player",      
        centerCoord: [0, 0, 0],      
        waitTicksForChunkLoad: 40,
        maxTries: 5,
        globalCooldownSec: 180
    },
    
    tpz: {
        dimsEnabled: [true, true, true],
        dimsCost: [100, 200, 300],
        dimsRadius: [5000, 3000, 2000]
    },
    
    warp: {
    },
    
    server: {
        enabled: true,
        permissionLevel: 1        
    },
    
    deathtp: {                     
        enabled: true,
        maxRecords: 5,            
        baseCost: 100,             
        costMultiplier: 1.5,       
        cooldownSec: 60,           
        invincibleEnabled: true,   
        invincibleDuration: 5,     
        resistanceLevel: 4         
    }
};

let config = {};
if (!File.exists(CONFIG_PATH)) {
    File.writeTo(CONFIG_PATH, JSON.stringify(defaultConfig, null, 4));
    config = { ...defaultConfig };
} else {
    try {
        let raw = File.readFrom(CONFIG_PATH);
        let loadedConfig = JSON.parse(raw);
        
        let oldConfigKeys = Object.keys(loadedConfig).length;
        config = mergeConfig(loadedConfig, defaultConfig);
        let newConfigKeys = Object.keys(config).length;
        
        let configChanged = newConfigKeys > oldConfigKeys;
        
        if (configChanged) {
            saveConfig();
            logger.info(`[${PLUGIN_NAME}] 配置文件已更新，添加了新的配置项`);
        }
    } catch (e) {
        logger.error(`[${PLUGIN_NAME}] Failed to read config: ${e}`);
        config = { ...defaultConfig };
    }
}

function mergeConfig(source, defaults) {
    if (!source) return { ...defaults };
    
    let result = { ...source };
    
    for (let key in defaults) {
        if (typeof defaults[key] === 'object' && !Array.isArray(defaults[key])) {
            result[key] = mergeConfig(source[key], defaults[key]);
        } else if (source[key] === undefined) {
            result[key] = defaults[key];
        }
    }
    
    return result;
}

let homesData = {};
if (!File.exists(HOME_DATA_PATH)) {
    File.writeTo(HOME_DATA_PATH, JSON.stringify({}, null, 4));
} else {
    try {
        let raw = File.readFrom(HOME_DATA_PATH);
        homesData = JSON.parse(raw);
    } catch (e) {
        logger.error(`[${PLUGIN_NAME}] Failed to read homes data: ${e}`);
        homesData = {};
    }
}

let warpsData = {};
if (!File.exists(WARP_DATA_PATH)) {
    File.writeTo(WARP_DATA_PATH, JSON.stringify({}, null, 4));
} else {
    try {
        let raw = File.readFrom(WARP_DATA_PATH);
        warpsData = JSON.parse(raw);
    } catch (e) {
        logger.error(`[${PLUGIN_NAME}] Failed to read warps data: ${e}`);
        warpsData = {};
    }
}

function getBalance(xuid) {
    if (config.economyType === "scoreboard") {
        let pl = mc.getPlayer(xuid);
        if (!pl) return 0;
        let objName = config.scoreboardObj || "money";
        if (!mc.getScoreObjective(objName)) {
            mc.newScoreObjective(objName, objName);
        }
        let sc = pl.getScore(objName);
        if (typeof sc !== "number" || isNaN(sc)) sc = 0;
        return sc;
    } else {
        let bal = money.get(xuid);
        if (!bal) return 0;
        return bal;
    }
}

function reduceMoney(xuid, amt) {
    if (config.economyType === "scoreboard") {
        let pl = mc.getPlayer(xuid);
        if (!pl) return false;
        let objName = config.scoreboardObj || "money";
        if (!mc.getScoreObjective(objName)) {
            mc.newScoreObjective(objName, objName);
        }
        let sc = pl.getScore(objName);
        if (typeof sc !== "number" || isNaN(sc)) sc = 0;
        if (sc < amt) return false;
        pl.setScore(objName, sc - amt);
        return true;
    } else {
        return money.reduce(xuid, amt);
    }
}

function addMoney(xuid, amt) {
    if (config.economyType === "scoreboard") {
        let pl = mc.getPlayer(xuid);
        if (!pl) return false;
        let objName = config.scoreboardObj || "money";
        if (!mc.getScoreObjective(objName)) {
            mc.newScoreObjective(objName, objName);
        }
        let sc = pl.getScore(objName);
        if (typeof sc !== "number" || isNaN(sc)) sc = 0;
        pl.setScore(objName, sc + amt);
        return true;
    } else {
        return money.add(xuid, amt);
    }
}

function getDimName(dimId) {
    switch (dimId) {
        case 0: return "§a主世界";
        case 1: return "§c地狱";
        case 2: return "§d末地";
        default: return `未知维度(${dimId})`;
    }
}

function saveConfig() {
    if (!File.exists(BASE_DIR)) {
        File.createDir(BASE_DIR);
    }
    File.writeTo(CONFIG_PATH, JSON.stringify(config, null, 4));
}

function saveHomesData() {
    if (!File.exists(BASE_DIR)) {
        File.createDir(BASE_DIR);
    }
    File.writeTo(HOME_DATA_PATH, JSON.stringify(homesData, null, 4));
}

function saveWarpsData() {
    if (!File.exists(BASE_DIR)) {
        File.createDir(BASE_DIR);
    }
    File.writeTo(WARP_DATA_PATH, JSON.stringify(warpsData, null, 4));
}

mc.regPlayerCmd("tps", "§a集成传送系统", (player, args) => {
    showMainMenu(player);
});

mc.regPlayerCmd("home", "§a家系统", (player, args) => {
    if (!config.modulesEnabled.home) {
        player.tell("§c家系统模块已被禁用");
        return;
    }
    showHomeMenu(player);
});

mc.regPlayerCmd("tpa", "§d互传系统", (player, args) => {
    if (!config.modulesEnabled.tpa) {
        player.tell("§c互传系统模块已被禁用");
        return;
    }
    showTpaMenu(player);
});

mc.regPlayerCmd("tpr", "§b随机传送", (player, args) => {
    if (!config.modulesEnabled.tpr) {
        player.tell("§c随机传送模块已被禁用");
        return;
    }
    showTprMenu(player);
});

mc.regPlayerCmd("tpz", "§c坐标传送", (player, args) => {
    if (!config.modulesEnabled.tpz) {
        player.tell("§c坐标传送模块已被禁用");
        return;
    }
    showTpzMenu(player);
});

mc.regPlayerCmd("warp", "§e公共传送点", (player, args) => {
    if (!config.modulesEnabled.warp) {
        player.tell("§c公共传送点模块已被禁用");
        return;
    }
    
    if (args.length > 0) {
        let warpName = args.join(" ").trim();
        if (warpName && warpsData[warpName]) {
            doWarpTeleport(player, warpName);
        } else {
            player.tell(`§c不存在传送点 [${warpName}]`);
        }
        return;
    }
    
    showWarpMenu(player);
});

mc.regPlayerCmd("tpserver", "§b跨服传送", (player, args) => {
    if (!config.modulesEnabled.server) {
        player.tell("§c跨服传送模块已被禁用");
        return;
    }
    
    if (config.server.permissionLevel > 0 && !player.isOP()) {
        player.tell("§c你没有权限使用跨服传送");
        return;
    }
    
    showServerMenu(player);
});

mc.regPlayerCmd("tpdeath", "§c死亡点传送", (player, args) => {
    if (!config.modulesEnabled.deathtp || !config.deathtp.enabled) {
        player.tell("§c死亡点传送模块已被禁用");
        return;
    }
    
    showDeathTpMenu(player);
});

function showMainMenu(player) {
    let fm = mc.newSimpleForm();
    fm.setTitle("§b集成传送系统");
    fm.setContent("§e请选择传送功能:");
    
    let buttons = [];
    
    if (config.modulesEnabled.home) {
        fm.addButton("§a家系统", "textures/ui/icon_recipe_nature");
        buttons.push("home");
    }
    
    if (config.modulesEnabled.tpa) {
        fm.addButton("§d互传系统", "textures/ui/dressing_room_skins");
        buttons.push("tpa");
    }
    
    if (config.modulesEnabled.tpr) {
        fm.addButton("§b随机传送", "textures/ui/icon_random");
        buttons.push("tpr");
    }
    
    if (config.modulesEnabled.tpz) {
        fm.addButton("§c坐标传送", "textures/ui/icon_map");
        buttons.push("tpz");
    }
    
    if (config.modulesEnabled.warp) {
        fm.addButton("§e公共传送点", "textures/ui/icon_multiplayer");
        buttons.push("warp");
    }
    
    if (config.modulesEnabled.server && config.server.enabled) {
        fm.addButton("§b跨服传送", "textures/ui/realms_key_art.png");
        buttons.push("server");
    }
    
    if (!config.modulesEnabled.deathtp) {
        config.modulesEnabled.deathtp = true;
        saveConfig();
    }
    
    if (!config.deathtp) {
        config.deathtp = { ...defaultConfig.deathtp };
        saveConfig();
    }
    
    if (config.modulesEnabled.deathtp && config.deathtp.enabled) {
        fm.addButton("§c死亡点传送", "textures/ui/icon_alex");
        buttons.push("deathtp");
    }
    
    if (player.isOP()) {
        fm.addButton("§3系统配置", "textures/ui/settings_glyph_color_2x");
        buttons.push("config");
    }
    
    player.sendForm(fm, (pl, id) => {
        if (id === null) return;
        
        let selected = buttons[id];
        switch(selected) {
            case "home":
                showHomeMenu(pl);
                break;
            case "tpa":
                showTpaMenu(pl);
                break;
            case "tpr":
                showTprMenu(pl);
                break;
            case "tpz":
                showTpzMenu(pl);
                break;
            case "warp":
                showWarpMenu(pl);
                break;
            case "server":
                showServerMenu(pl);
                break;
            case "deathtp":
                showDeathTpMenu(pl);
                break;
            case "config":
                showConfigMenu(pl);
                break;
        }
    });
}

//---------------------------------------------------------------HOME
const pendingHomeRequests = {};

function showHomeMenu(player) {
    let fm = mc.newSimpleForm();
    fm.setTitle("§a家系统");
    fm.setContent("§e请选择功能:");
    fm.addButton("§b传送到家", "textures/ui/icon_map");
    fm.addButton("§d设置家", "textures/ui/anvil_icon");
    if (player.isOP()) {
        fm.addButton("§c配置家系统", "textures/ui/op");
    }
    
    player.sendForm(fm, (pl, id) => {
        if (id === null) return;
        
        if (id === 0) {
            showHomeListForm(pl);
        } else if (id === 1) {
            showSetHomeMenu(pl);
        } else if (pl.isOP() && id === 2) {
            showHomeConfigForm(pl);
        }
    });
}

function getPlayerHomes(xuid) {
    let arr = homesData[xuid];
    if (!Array.isArray(arr)) arr = [];
    return arr;
}

function setPlayerHomes(xuid, arr) {
    homesData[xuid] = arr;
    saveHomesData();
}

function showHomeListForm(player) {
    let arr = getPlayerHomes(player.xuid);
    if (arr.length === 0) {
        player.tell("§c你尚未设置任何家.");
        return;
    }
    
    let fm = mc.newSimpleForm();
    fm.setTitle("§b传送到家");
    fm.setContent("§e请选择要传送的家:");
    
    let cd = config.home.teleportCooldownSec || 60;
    let now = Date.now();
    
    arr.forEach(home => {
        let line1 = `§a${home.name} §f(${Math.floor(home.x)},${Math.floor(home.y)},${Math.floor(home.z)})`;
        let usedAgo = Math.floor((now - home.lastUse) / 1000);
        let remain = cd - usedAgo;
        if (remain < 0) remain = 0;
        let dimName = getDimName(home.dim);
        let line2 = `§b${dimName} §f| §c冷却剩余:${remain}s`;
        fm.addButton(line1 + "\n" + line2);
    });
    
    player.sendForm(fm, (pl, id) => {
        if (id === null) return;
        if (id >= 0 && id < arr.length) {
            let chosen = arr[id];
            teleportToHome(pl, chosen);
        }
    });
}

function teleportToHome(player, homeObj) {
    let xuid = player.xuid;
    let dimId = homeObj.dim;
    
    if (!config.home.dimsEnabled[dimId]) {
        player.tell("§c该维度家已被禁用,无法传送.");
        return;
    }
    
    let cdSec = config.home.teleportCooldownSec || 60;
    let now = Date.now();
    let remain = Math.floor(cdSec - (now - homeObj.lastUse) / 1000);
    
    if (remain > 0) {
        player.tell(`§c该家的传送仍在冷却,剩余${remain}秒`);
        return;
    }
    
    let cost = config.home.dimsCost[dimId] || 0;
    if (cost > 0) {
        let bal = getBalance(xuid);
        if (bal < cost) {
            player.tell(`§c传送需要${cost}, 余额不足(${bal}).`);
            return;
        }
        
        let ok = reduceMoney(xuid, cost);
        if (!ok) {
            player.tell(`§c扣费失败,可能余额不足.`);
            return;
        }
    }
    
    let newPos = new FloatPos(homeObj.x, homeObj.y, homeObj.z, dimId);
    let success = player.teleport(newPos);
    
    if (success) {
        player.tell(`§a成功传送到家[${homeObj.name}]! 扣费:${cost}`);
        playTeleportSound(player, "home");  
        homeObj.lastUse = Date.now();
        
        let arr = getPlayerHomes(xuid);
        for (let i = 0; i < arr.length; i++) {
            if (arr[i].name === homeObj.name && arr[i].dim === dimId) {
                arr[i] = homeObj;
                break;
            }
        }
        
        setPlayerHomes(xuid, arr);
    } else {
        player.tell(`§c传送失败,请检查坐标/维度是否可达.`);
    }
}

function showSetHomeMenu(player) {
    let fm = mc.newSimpleForm();
    fm.setTitle("§d设置家");
    fm.setContent("§e请选择操作:");
    fm.addButton("§a添加家", "textures/ui/icon_new");
    fm.addButton("§c删除家", "textures/ui/trash");
    fm.addButton("§b修改家", "textures/ui/touch_glyph");
    
    player.sendForm(fm, (pl, id) => {
        if (id === null) return;
        
        if (id === 0) {
            addHomeFlow(pl);
        } else if (id === 1) {
            removeHomeFlow(pl);
        } else if (id === 2) {
            renameHomeFlow(pl);
        }
    });
}

function addHomeFlow(player) {
    let dimid = player.pos.dimid;
    if (!config.home.dimsEnabled[dimid]) {
        player.tell("§c当前维度已禁用设置家.");
        return;
    }
    
    let arr = getPlayerHomes(player.xuid);
    let maxH = config.home.maxHomes || 3;
    
    if (arr.length >= maxH) {
        player.tell(`§c你的家数量已达上限(${maxH}).`);
        return;
    }
    
    let base = config.home.baseSetCost || 100;
    let inc = config.home.incSetCost || 50;
    let nth = arr.length;
    let cost = base + (inc * nth);
    
    let fm = mc.newCustomForm();
    fm.setTitle("§a添加家");
    fm.addLabel(`§b你当前位置: x=${Math.floor(player.pos.x)} y=${Math.floor(player.pos.y)} z=${Math.floor(player.pos.z)}\n` +
                `§e本次设置家花费: ${cost}`);
    fm.addInput("家名称(不超过10字符)", "Home1", "");
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let newName = (data[1] || "").trim();
        if (!newName || newName.length > 10) {
            pl.tell("§c家名不合法(不能为空且不超过10字符).");
            return;
        }
        
        let xuid = pl.xuid;
        let bal = getBalance(xuid);
        
        if (bal < cost) {
            pl.tell(`§c余额不足,需要${cost},当前${bal}.`);
            return;
        }
        
        let ok = reduceMoney(xuid, cost);
        if (!ok) {
            pl.tell("§c扣费失败,请重试.");
            return;
        }
        
        let newHome = {
            name: newName,
            x: pl.pos.x,
            y: pl.pos.y,
            z: pl.pos.z,
            dim: pl.pos.dimid,
            lastUse: 0
        };
        
        let arr2 = getPlayerHomes(xuid);
        arr2.push(newHome);
        setPlayerHomes(xuid, arr2);
        
        pl.tell(`§a已成功在此位置添加家[${newName}]! 扣费:${cost}`);
    });
}

function removeHomeFlow(player) {
    let arr = getPlayerHomes(player.xuid);
    if (arr.length === 0) {
        player.tell("§c你没有任何家可删除.");
        return;
    }
    
    let names = arr.map(h => h.name);
    let fm = mc.newCustomForm();
    fm.setTitle("§c删除家");
    fm.addDropdown("选择要删除的家:", names, 0);
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let selIdx = data[0];
        if (selIdx < 0 || selIdx >= arr.length) {
            pl.tell("§c选择无效");
            return;
        }
        
        let home = arr[selIdx];
        let fm2 = mc.newSimpleForm();
        fm2.setTitle("§c确认删除");
        fm2.setContent(`§e确定删除家[${home.name}]?\n此操作不退费!`);
        fm2.addButton("§c确认删除");
        fm2.addButton("§a取消");
        
        pl.sendForm(fm2, (pl2, id2) => {
            if (id2 === 0) {
                arr.splice(selIdx, 1);
                setPlayerHomes(pl2.xuid, arr);
                pl2.tell(`§a已删除家[${home.name}]!`);
            }
        });
    });
}

function renameHomeFlow(player) {
    let arr = getPlayerHomes(player.xuid);
    if (arr.length === 0) {
        player.tell("§c你没有任何家可修改.");
        return;
    }
    
    let names = arr.map(h => h.name);
    let fm = mc.newCustomForm();
    fm.setTitle("§b修改家名称");
    fm.addDropdown("选择要修改的家:", names, 0);
    fm.addInput("新的家名称(<=10字符)", "HomeX", "");
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let selIdx = data[0];
        let newName = (data[1] || "").trim();
        
        if (selIdx < 0 || selIdx >= arr.length) {
            pl.tell("§c选择无效.");
            return;
        }
        
        if (!newName || newName.length > 10) {
            pl.tell("§c新家名不合法.");
            return;
        }
        
        arr[selIdx].name = newName;
        setPlayerHomes(pl.xuid, arr);
        pl.tell(`§a已重命名家为: ${newName}`);
    });
}

function showHomeConfigForm(player) {
    let fm = mc.newCustomForm();
    fm.setTitle("§c配置家系统");
    
    fm.addInput("§a每人最大家数", "", String(config.home.maxHomes || 3));
    fm.addInput("§a初次设置家费用", "", String(config.home.baseSetCost || 100));
    fm.addInput("§c每次添加一个家的递增费用", "", String(config.home.incSetCost || 50));
    fm.addSwitch("§a启用主世界?", config.home.dimsEnabled[0]);
    fm.addSwitch("§c启用地狱?", config.home.dimsEnabled[1]);
    fm.addSwitch("§d启用末地?", config.home.dimsEnabled[2]);
    fm.addInput("§a主世界传送额外费用", "", String(config.home.dimsCost[0] || 0));
    fm.addInput("§c地狱传送额外费用", "", String(config.home.dimsCost[1] || 0));
    fm.addInput("§d末地传送额外费用", "", String(config.home.dimsCost[2] || 0));
    fm.addInput("§s传送冷却(秒)", "60", String(config.home.teleportCooldownSec || 60));
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let idx = 0;
        let maxH = parseInt(data[idx++]);
        let baseC = parseInt(data[idx++]);
        let incC = parseInt(data[idx++]);
        let e0 = data[idx++];
        let e1 = data[idx++];
        let e2 = data[idx++];
        let c0 = parseInt(data[idx++]);
        let c1 = parseInt(data[idx++]);
        let c2 = parseInt(data[idx++]);
        let cdSec = parseInt(data[idx++]);
        
        if (isNaN(maxH) || maxH < 1) maxH = 1;
        if (isNaN(baseC) || baseC < 0) baseC = 0;
        if (isNaN(incC) || incC < 0) incC = 0;
        if (isNaN(c0) || c0 < 0) c0 = 0;
        if (isNaN(c1) || c1 < 0) c1 = 0;
        if (isNaN(c2) || c2 < 0) c2 = 0;
        if (isNaN(cdSec) || cdSec < 0) cdSec = 60;
        
        config.home.maxHomes = maxH;
        config.home.baseSetCost = baseC;
        config.home.incSetCost = incC;
        config.home.dimsEnabled = [e0, e1, e2];
        config.home.dimsCost = [c0, c1, c2];
        config.home.teleportCooldownSec = cdSec;
        
        saveConfig();
        
        pl.tell("§a已更新家系统配置!");
    });
}

//---------------------------------------------------------------TPA
const pendingTpaRequests = {};

function showTpaMenu(player) {
    let onlinePlayers = mc.getOnlinePlayers().filter(p => p.name !== player.name);
    if (onlinePlayers.length === 0) {
        player.tell("§c当前没有其他在线玩家。");
        return;
    }
    
    let form = mc.newCustomForm();
    form.setTitle("§a互传请求");
    let nameList = onlinePlayers.map(p => p.name);
    form.addDropdown("§s选择目标玩家", nameList);
    form.addDropdown("§a传送方式", ["§a传送到对方", "§e让对方传送到我"]);
    
    let isDelayEnabled = (config.tpa.isDelayEnabled !== false);
    let maxD = Number(config.tpa.maxDelay) || 20;
    let addedDelaySlider = false;
    
    if (isDelayEnabled) {
        form.addSlider(`§e传送延迟(0~${maxD}秒)`, 0, maxD, 1, 0);
        addedDelaySlider = true;
    }
    
    let isOp = player.isOP();
    if (isOp) {
        form.addSwitch("§c管理互传(开启后进入管理页面)", false);
    }
    
    player.sendForm(form, (pl, data) => {
        if (!data) {
            pl.tell("§c操作已取消。");
            return;
        }
        
        let idx = 0;
        let targetIndex = data[idx++];
        let modeIndex = data[idx++];
        
        let delaySec = 0;
        if (addedDelaySlider) {
            delaySec = data[idx++];
        }
        
        let manage = false;
        if (isOp) {
            manage = data[idx++];
        }
        
        if (manage === true) {
            showTpaManageForm(pl);
            return;
        }
        
        let targetName = nameList[targetIndex];
        let direction = (modeIndex === 0 ? "to" : "here");
        sendTpaRequest(pl, targetName, direction, Math.floor(delaySec));
    });
}

function showTpaManageForm(player) {
    let form = mc.newCustomForm();
    form.setTitle("§c管理互传");
    form.addInput("§e请求超时时间(秒)", "数字", "" + config.tpa.requestTimeout);
    form.addDropdown("§e请求提示方式", ["form(对方弹窗)", "bossbar(血条)"], (config.tpa.promptType === "bossbar" ? 1 : 0));
    let isDelayOn = (config.tpa.isDelayEnabled !== false);
    form.addSwitch("§b是否启用延迟功能", isDelayOn);
    form.addInput("§b最大延迟秒数", "数字", "" + (config.tpa.maxDelay || 20));
    
    player.sendForm(form, (pl, data) => {
        if (!data) {
            pl.tell("§c已取消操作");
            return;
        }
        
        let newTimeout = parseInt(data[0]);
        let promptIndex = data[1];
        let enableDelay = data[2];
        let newMaxDelay = parseInt(data[3]);
        
        if (isNaN(newTimeout) || newTimeout <= 0) {
            pl.tell("§c请求超时必须是正整数！");
            return;
        }
        
        if (isNaN(newMaxDelay) || newMaxDelay < 0) {
            pl.tell("§c最大延迟必须>=0！");
            return;
        }
        
        let newPromptType = (promptIndex === 0 ? "form" : "bossbar");
        
        config.tpa.requestTimeout = newTimeout;
        config.tpa.promptType = newPromptType;
        config.tpa.isDelayEnabled = enableDelay;
        config.tpa.maxDelay = newMaxDelay;
        
        saveConfig();
        
        pl.tell(`§a配置已更新: 超时=${newTimeout}, 提示=${newPromptType}, 延迟开关=${enableDelay}, maxDelay=${newMaxDelay}`);
    });
}

function sendTpaRequest(fromPlayer, toPlayerName, direction, delaySec) {
    let toPlayer = mc.getPlayer(toPlayerName);
    if (!toPlayer) {
        fromPlayer.tell(`§c目标玩家[${toPlayerName}]已离线,无法发送请求`);
        return;
    }
    
    if (pendingTpaRequests[toPlayerName]) {
        fromPlayer.tell("§c对方已有未处理的请求, 稍后再试");
        return;
    }
    
    let uid = Math.floor(Math.random() * 1e8);
    let req = {
        from: fromPlayer,
        to: toPlayer,
        fromName: fromPlayer.name,
        toName: toPlayerName,
        direction: direction,
        delay: delaySec,
        bossbarId: uid,
        startTime: Date.now()
    };
    
    pendingTpaRequests[toPlayerName] = req;
    
    let pType = config.tpa.promptType || "form";
    let timeoutSec = config.tpa.requestTimeout || 60;
    
    toPlayer.tell(`§e收到传送请求(${req.fromName}想${direction === "to" ? "传送到你" : "让你传送到他"})\n` +
                 (delaySec > 0 ? `§6并设置了延迟: ${delaySec}秒\n` : "") +
                 `§e可在表单点击【同意/拒绝】或输入 /tpy(同意)/tpn(拒绝)\n` +
                 `§c请求最多等待${timeoutSec}秒`);
    
    fromPlayer.tell(`§a已向 ${toPlayerName} 发送请求(延迟=${delaySec}), 等待对方同意(最多${timeoutSec}s)`);
    
    if (pType === "form") {
        showTpaConfirmForm(req, timeoutSec);
    } else {
        showTpaBossbarPrompt(req, timeoutSec);
    }
}

function showTpaConfirmForm(req, timeoutSec) {
    let toPlayer = req.to;
    let fromName = req.fromName;
    let dirText = (req.direction === "to" ? "传送到你" : "让你传送到他");
    let delayStr = (req.delay > 0 ? `(延迟${req.delay}秒)\n` : "");
    
    let form = mc.newSimpleForm();
    form.setTitle("§a传送请求");
    form.setContent(`§b[${fromName}] 请求${dirText}\n` +
                   `${delayStr}` +
                   `§f(可输入 /tpy 或 /tpn)\n` +
                   `§e剩余时间: ${timeoutSec}s`);
    form.addButton("§a同意");
    form.addButton("§c拒绝");
    
    toPlayer.sendForm(form, (pl, id) => {
        if (id == null) return; 
        if (id === 0) acceptTpaRequest(pl.name);
        else denyTpaRequest(pl.name);
    });
    
    startTpaRequestCountdown(req, timeoutSec, false);
}

function showTpaBossbarPrompt(req, timeoutSec) {
    let toPlayer = req.to;
    let fromName = req.fromName;
    let dirText = (req.direction === "to" ? "传送到你" : "让你传送到他");
    let delayStr = (req.delay > 0 ? `(延迟${req.delay}秒)` : "");
    let barId = req.bossbarId;
    
    toPlayer.setBossBar(barId,
        `§a${fromName}请求${dirText}§f${delayStr}\n§c(/tpy同意 /tpn拒绝)`,
        100, 3
    );
    
    startTpaRequestCountdown(req, timeoutSec, true);
}

function startTpaRequestCountdown(req, timeoutSec, bossbarMode) {
    let remain = timeoutSec;
    let to = req.to;
    let from = req.from;
    let barId = req.bossbarId;
    
    let timerId = setInterval(() => {
        remain--;
        
        if (!mc.getPlayer(to.name) || !mc.getPlayer(from.name)) {
            clearInterval(timerId);
            cancelTpaRequest(to.name, "§c对方(或你)下线,请求取消");
            return;
        }
        
        if (bossbarMode) {
            let percent = Math.floor((remain / timeoutSec) * 100);
            let dirText = (req.direction === "to" ? "传送到你" : "让你传送到他");
            let delayStr = (req.delay > 0 ? `(延迟${req.delay}秒)` : "");
            to.setBossBar(barId,
                `§a${from.name}请求${dirText}§f${delayStr}§s(/tpy同意 /tpn拒绝),剩余${remain}s`,
                percent, 3
            );
        }
        
        if (remain <= 0) {
            clearInterval(timerId);
            cancelTpaRequest(to.name, "§c传送请求已超时");
        }
    }, 1000);
    
    req.timer = timerId;
}

mc.regPlayerCmd("tpy", "§a同意传送请求", (pl) => {
    acceptTpaRequest(pl.name);
});

mc.regPlayerCmd("tpn", "§c拒绝传送请求", (pl) => {
    denyTpaRequest(pl.name);
});


function acceptTpaRequest(targetName) {
    let req = pendingTpaRequests[targetName];
    if (!req) {
        let p = mc.getPlayer(targetName);
        if (p) p.tell("§c你没有待处理的请求。");
        return;
    }
    
    clearTpaRequest(req);
    
    let from = req.from, to = req.to;
    let delay = req.delay;
    let dir = req.direction;
    
    to.tell("§a已同意传送请求。");
    from.tell(`§a对方已同意请求，` + (delay > 0 ? `将在${delay}秒后传送...` : "正在传送..."));
    
    if (delay > 0) {
        let secondBarId = Math.floor(Math.random() * 1e9);
        let remain = delay;
        
        let secondTid = setInterval(() => {
            remain--;
            
            if (!mc.getPlayer(from.name) || !mc.getPlayer(to.name)) {
                clearInterval(secondTid);
                from.removeBossBar(secondBarId);
                to.removeBossBar(secondBarId);
                from.tell("§c传送中断，对方或你下线。");
                return;
            }
            
            let percent = Math.floor((remain / delay) * 100);
            from.setBossBar(secondBarId, `§d传送倒计时: ${remain}s`, percent, 1);
            to.setBossBar(secondBarId, `§d传送倒计时: ${remain}s`, percent, 1);
            
            if (remain <= 0) {
                clearInterval(secondTid);
                from.removeBossBar(secondBarId);
                to.removeBossBar(secondBarId);
                
                if (dir === "to") {
                    let targetPlayer = mc.getPlayer(to.name);
                    if (!targetPlayer) {
                        from.tell("§c传送失败，目标玩家已离线。");
                        return;
                    }
                    let footPos = new FloatPos(
                        targetPlayer.pos.x,
                        targetPlayer.pos.y - 1.62, 
                        targetPlayer.pos.z,
                        targetPlayer.pos.dimid
                    );
                    from.teleport(footPos);
                } else {
                    let targetPlayer = mc.getPlayer(from.name);
                    if (!targetPlayer) {
                        to.tell("§c传送失败，目标玩家已离线。");
                        return;
                    }
                    let footPos = new FloatPos(
                        targetPlayer.pos.x,
                        targetPlayer.pos.y - 1.62, 
                        targetPlayer.pos.z,
                        targetPlayer.pos.dimid
                    );
                    to.teleport(footPos);
                }
                
                from.tell("§a传送成功!");
                to.tell("§a传送成功!");
                playTeleportSound(from, "tpa");
            }
        }, 1000);
    } else {
        if (!mc.getPlayer(from.name) || !mc.getPlayer(to.name)) {
            from.tell("§c传送中断，对方(或你)下线。");
            return;
        }
        
        if (dir === "to") {
            let targetPlayer = mc.getPlayer(to.name);
            if (!targetPlayer) {
                from.tell("§c传送失败，目标玩家已离线。");
                return;
            }
            let footPos = new FloatPos(
                targetPlayer.pos.x,
                targetPlayer.pos.y - 1.62, 
                targetPlayer.pos.z,
                targetPlayer.pos.dimid
            );
            from.teleport(footPos);
        } else {
            let targetPlayer = mc.getPlayer(from.name);
            if (!targetPlayer) {
                to.tell("§c传送失败，目标玩家已离线。");
                return;
            }
            let footPos = new FloatPos(
                targetPlayer.pos.x,
                targetPlayer.pos.y - 1.62, 
                targetPlayer.pos.z,
                targetPlayer.pos.dimid
            );
            to.teleport(footPos);
        }
        
        from.tell("§a传送成功!");
        to.tell("§a传送成功!");
    }
    
    delete pendingTpaRequests[targetName];
}

function denyTpaRequest(targetName) {
    let req = pendingTpaRequests[targetName];
    if (!req) {
        let p = mc.getPlayer(targetName);
        if (p) p.tell("§c你没有待处理的请求。");
        return;
    }
    
    clearTpaRequest(req);
    req.from.tell("§c对方拒绝了传送请求。");
    req.to.tell("§e你已拒绝传送请求。");
    delete pendingTpaRequests[targetName];
}

function cancelTpaRequest(targetName, msg) {
    let req = pendingTpaRequests[targetName];
    if (!req) return;
    
    clearTpaRequest(req);
    req.from.tell(msg);
    delete pendingTpaRequests[targetName];
}

function clearTpaRequest(req) {
    if (req.timer) {
        clearInterval(req.timer);
    }
    
    let to = req.to;
    if (to && mc.getPlayer(to.name)) {
        to.removeBossBar(req.bossbarId);
    }
}

mc.listen("onLeft", (pl) => {
    let pname = pl.name;
    
    for (let [key, request] of Object.entries(pendingTpaRequests)) {
        if (!request || !request.from || !request.to) continue;
        
        if (request.toName === pname) {
            clearTpaRequest(request);
            request.from.tell("§c对方下线，传送请求取消。");
            delete pendingTpaRequests[key];
        } else if (request.fromName === pname) {
            let rec = request.to;
            if (rec) rec.tell("§c请求方下线，传送请求取消。");
            clearTpaRequest(request);
            delete pendingTpaRequests[key];
        }
    }
});

//---------------------------------------------------------------TPR
const lastTprUse = {};
const tprSearchingTaskMap = {};

function showTprMenu(player) {
    let fm = mc.newCustomForm();
    fm.setTitle("§a随机传送");
    
    let owE = config.tpr.dimsEnabled[0];
    let ntE = config.tpr.dimsEnabled[1];
    let edE = config.tpr.dimsEnabled[2];
    let owR = config.tpr.maxRangePerDim[0] || 3000;
    let ntR = config.tpr.maxRangePerDim[1] || 1000;
    let edR = config.tpr.maxRangePerDim[2] || 1000;
    let owC = config.tpr.costPerDim[0] || 0;
    let ntC = config.tpr.costPerDim[1] || 0;
    let edC = config.tpr.costPerDim[2] || 0;
    
    let centerMethod = config.tpr.centerMethod || "player";
    let centerText = centerMethod === "player" ? "以玩家为中心" : "以坐标为中心";
    if (centerMethod === "coord") {
        let x = config.tpr.centerCoord ? config.tpr.centerCoord[0] : 0;
        let z = config.tpr.centerCoord ? config.tpr.centerCoord[2] : 0;
        centerText += ` (${x}, ${z})`;
    }
    
    let line =
        `§a主世界:${owE ? "§a(已启用)" : "§7(未启用)"} §f范围:${owR},费:${owC}\n` +
        `§c地狱:${ntE ? "§c(已启用)" : "§7(未启用)"} §f范围:${ntR},费:${ntC}\n` +
        `§d末地:${edE ? "§d(已启用)" : "§7(未启用)"} §f范围:${edR},费:${edC}\n` +
        `§b${centerText}\n` +
        `§e全局冷却:${config.tpr.globalCooldownSec || 180}s`;
    
    fm.addLabel(line);
    
    let dims = [
        `${owE ? "§a" : "§7"}主世界${owE ? "" : "(未启用)"}`,
        `${ntE ? "§c" : "§7"}地狱${ntE ? "" : "(未启用)"}`,
        `${edE ? "§d" : "§7"}末地${edE ? "" : "(未启用)"}`
    ];
    
    fm.addDropdown("选择维度", dims, 0);
    fm.addSwitch("确认传送?", false);
    
    let isOp = player.isOP();
    if (isOp) {
        fm.addSwitch("§c进入管理员配置?", false);
    }
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let idx = 0;
        let dimIdx = data[++idx];
        let goTrans = data[++idx];
        let goConfig = false;
        
        if (isOp) {
            goConfig = data[++idx];
        }
        
        if (goConfig) {
            showTprConfigForm(pl);
            return;
        }
        
        if (!goTrans) {
            pl.tell("§c取消传送.");
            return;
        }
        
        let xuid = pl.xuid;
        let cdCheck = canUseTprGlobalCD(xuid);
        
        if (!cdCheck.ok) {
            pl.tell(`§c距离下次随机传送还有 ${cdCheck.remain}s`);
            return;
        }
        
        if (!config.tpr.dimsEnabled[dimIdx]) {
            pl.tell("§c该维度已禁用.");
            return;
        }
        
        setTprGlobalCD(xuid);
        doRandomTeleport(pl, dimIdx);
    });
}

function canUseTprGlobalCD(xuid) {
    let cdSec = config.tpr.globalCooldownSec || 180;
    let last = lastTprUse[xuid] || 0;
    let now = Date.now();
    let passed = Math.floor((now - last) / 1000);
    let remain = cdSec - passed;
    
    if (remain > 0) {
        return { ok: false, remain: remain };
    }
    
    return { ok: true, remain: 0 };
}

function setTprGlobalCD(xuid) {
    lastTprUse[xuid] = Date.now();
}

function doRandomTeleport(player, dim) {
    let cost = 0, range = 1000, yMin = 0, yMax = 255;
    
    switch (dim) {
        case 0:
            cost = config.tpr.costPerDim[0] || 0;
            range = config.tpr.maxRangePerDim[0] || 3000;
            yMin = -60; yMax = 320;
            break;
        case 1:
            cost = config.tpr.costPerDim[1] || 0;
            range = config.tpr.maxRangePerDim[1] || 1000;
            yMin = 0; yMax = 120;
            break;
        case 2:
            cost = config.tpr.costPerDim[2] || 0;
            range = config.tpr.maxRangePerDim[2] || 1000;
            yMin = 0; yMax = 200;
            break;
    }
    
    if (cost > 0) {
        let xuid = player.xuid;
        let bal = getBalance(xuid);
        
        if (bal < cost) {
            player.tell(`§c余额不足,需要${cost},当前${bal}`);
            return;
        }
        
        let ok = reduceMoney(xuid, cost);
        if (!ok) {
            player.tell("§c扣费失败或余额不足.");
            return;
        }
    }
    
    let oldPos = player.pos;
    let centerMethod = config.tpr.centerMethod || "player";
    let cx = 0, cz = 0;
    
    if (centerMethod === "player") {
        cx = Math.floor(player.pos.x);
        cz = Math.floor(player.pos.z);
    } else {
        cx = config.tpr.centerCoord[0] || 0;
        cz = config.tpr.centerCoord[2] || 0;
    }
    
    let tries = 0;
    let maxTries = config.tpr.maxTries || 5;
    let waitTicks = config.tpr.waitTicksForChunkLoad || 40;
    
    let xuid = player.xuid || "NO_XUID";
    startTprSearchEffect(player);
    
    function attemptOne() {
        tries++;
        let rx, rz;
        
        if (dim === 2) {
            let validPoint = false;
            while (!validPoint) {
                rx = cx + Math.floor(Math.random() * (range * 2 + 1) - range);
                rz = cz + Math.floor(Math.random() * (range * 2 + 1) - range);
                
                let distToCenter = Math.sqrt(rx * rx + rz * rz);
                
                if (distToCenter < 200 || distToCenter > 1000) {
                    validPoint = true;
                }
            }
        } else {
            rx = cx + Math.floor(Math.random() * (range * 2 + 1) - range);
            rz = cz + Math.floor(Math.random() * (range * 2 + 1) - range);
        }
    
        let initialY;
        if (dim === 0) {
            initialY = 230;//主世界初始高度
        } else if (dim === 1) {
            initialY = 130;//地狱初始高度
        } else if (dim === 2) {
            initialY = 220;//末地初始高度
        } else {
            initialY = yMin + 1;
        }
        let stepPos = new FloatPos(rx, initialY, rz, dim);
        let stOk = player.teleport(stepPos);
        
        if (!stOk) {
            if (tries >= maxTries) {
                if (cost > 0) addMoney(xuid, cost);
                player.teleport(oldPos);
                player.tell("§c随机传送失败(无法传到初始位置).");
                stopTprSearchEffect(xuid);
                return;
            } else {
                attemptOne();
                return;
            }
        }
        
        setTimeout(() => {
            let found = false;
            
            if (dim === 1) { 
                found = searchSafeBlockUp(player, rx, yMin, yMax, rz, dim);
            } else { 
                found = searchSafeBlockDown(player, rx, yMin, yMax, rz, dim);
            }
            
            if (found) {
                player.tell("§a随机传送成功!");
                playTeleportSound(player, "tpr");
                stopTprSearchEffect(xuid);
            } else {
                if (tries >= maxTries) {
                    if (cost > 0) addMoney(xuid, cost);
                    player.teleport(oldPos);
                    player.tell("§c随机传送失败,未找到安全落点.");
                    stopTprSearchEffect(xuid);
                } else {
                    attemptOne();
                }
            }
        }, waitTicks * 50);
    }
    
    attemptOne();
}

function searchSafeBlockUp(player, x, bottomY, topY, z, dim) {
    if (bottomY < 0) bottomY = 0;
    
    for (let y = bottomY; y <= topY - 2; y++) {
        let blockBelow = mc.getBlock(x, y, z, dim);
        let blockPos = mc.getBlock(x, y + 1, z, dim);
        let blockAbove = mc.getBlock(x, y + 2, z, dim);  
        
        if (!blockBelow || !blockPos || !blockAbove) {
            continue;
        }
        
        let bName = blockBelow.type || "";
        let pName = blockPos.type || "";
        let aName = blockAbove.type || "";
        
        if (isSolidBlock(bName) && isAirBlock(pName) && isAirBlock(aName)) {
            let finalPos = new FloatPos(x, y + 1, z, dim);
            let ok = player.teleport(finalPos);
            return ok;
        }
    }
    
    return false;
}
function searchSafeBlockDown(player, x, bottomY, topY, z, dim) {
    for (let y = topY; y >= bottomY + 1; y--) {
        let blockBelow = mc.getBlock(x, y - 1, z, dim);
        let blockPos = mc.getBlock(x, y, z, dim);
        let blockAbove = mc.getBlock(x, y + 1, z, dim);  
        
        if (!blockBelow || !blockPos || !blockAbove) {
            continue;
        }
        
        let bName = blockBelow.type || "";
        let pName = blockPos.type || "";
        let aName = blockAbove.type || "";
        
        if (isSolidBlock(bName) && isAirBlock(pName) && isAirBlock(aName)) {
            let finalPos = new FloatPos(x, y, z, dim);
            let ok = player.teleport(finalPos);
            return ok;
        }
    }
    
    return false;
}

function isSolidBlock(name) {
    if (!name) return false;
    if (name.includes("air") || name.includes("water") || name.includes("lava")) return false;
    return true;
}

function isAirBlock(name) {
    if (!name) return true;
    return (name.includes("air"));
}

function startTprSearchEffect(player) {
    let xuid = player.xuid;
    if (!xuid) return;
    
    if (tprSearchingTaskMap[xuid]) {
        return;
    }
    
    player.tell("§e正在寻找安全落脚点,请稍候...§a此过程免疫§c伤害");
    applyResistanceEffect(player);
    
    let tid = setInterval(() => {
        let pl = mc.getPlayer(xuid);
        if (!pl) {
            clearInterval(tid);
            delete tprSearchingTaskMap[xuid];
            return;
        }
        
        applyResistanceEffect(pl);
    }, 5000);
    
    tprSearchingTaskMap[xuid] = { intervalId: tid };
}

function stopTprSearchEffect(xuid) {
    let info = tprSearchingTaskMap[xuid];
    if (info) {
        clearInterval(info.intervalId);
        delete tprSearchingTaskMap[xuid];
    }
    
    let pl = mc.getPlayer(xuid);
    if (pl) {
        pl.removeEffect(11);
    }
}

function applyResistanceEffect(player) {
    player.addEffect(11, 6 * 20, 49, false);
}

function showTprConfigForm(player) {
    let fm = mc.newCustomForm();
    fm.setTitle("§c随机传送配置");
    
    fm.addSwitch("§a启用主世界?", config.tpr.dimsEnabled[0]);
    fm.addSwitch("§c启用地狱?", config.tpr.dimsEnabled[1]);
    fm.addSwitch("§d启用末地?", config.tpr.dimsEnabled[2]);
    
    fm.addInput("§a主世界费用", "", String(config.tpr.costPerDim[0] || 0));
    fm.addInput("§c地狱费用", "", String(config.tpr.costPerDim[1] || 0));
    fm.addInput("§d末地费用", "", String(config.tpr.costPerDim[2] || 0));
    
    fm.addInput("§a主世界范围", "", String(config.tpr.maxRangePerDim[0] || 3000));
    fm.addInput("§c地狱范围", "", String(config.tpr.maxRangePerDim[1] || 1000));
    fm.addInput("§d末地范围", "", String(config.tpr.maxRangePerDim[2] || 1000));
    
    let centerOptions = ["以玩家为中心", "以固定坐标为中心"];
    let centerIdx = (config.tpr.centerMethod === "coord" ? 1 : 0);
    fm.addDropdown("§b随机中心选择", centerOptions, centerIdx);
    
    let centerX = config.tpr.centerCoord ? config.tpr.centerCoord[0] : 0;
    let centerZ = config.tpr.centerCoord ? config.tpr.centerCoord[2] : 0;
    fm.addInput("§b中心坐标 X", "0", String(centerX));
    fm.addInput("§b中心坐标 Z", "0", String(centerZ));
    
    fm.addInput("§s等待区块加载Tick", "", String(config.tpr.waitTicksForChunkLoad || 40));
    fm.addInput("§s最大尝试次数", "", String(config.tpr.maxTries || 5));
    
    fm.addInput("§b全局冷却(秒)", "", String(config.tpr.globalCooldownSec || 180));
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let idx = 0;
        let e0 = data[idx++];
        let e1 = data[idx++];
        let e2 = data[idx++];
        let c0 = parseInt(data[idx++]);
        let c1 = parseInt(data[idx++]);
        let c2 = parseInt(data[idx++]);
        let r0 = parseInt(data[idx++]);
        let r1 = parseInt(data[idx++]);
        let r2 = parseInt(data[idx++]);
        let centerMethodIdx = data[idx++];
        let centerX = parseFloat(data[idx++]);
        let centerZ = parseFloat(data[idx++]);
        let wTicks = parseInt(data[idx++]);
        let mTry = parseInt(data[idx++]);
        let gCD = parseInt(data[idx++]);
        
        if (isNaN(c0) || c0 < 0) c0 = 0;
        if (isNaN(c1) || c1 < 0) c1 = 0;
        if (isNaN(c2) || c2 < 0) c2 = 0;
        if (isNaN(r0) || r0 < 0) r0 = 3000;
        if (isNaN(r1) || r1 < 0) r1 = 1000;
        if (isNaN(r2) || r2 < 0) r2 = 1000;
        if (isNaN(centerX)) centerX = 0;
        if (isNaN(centerZ)) centerZ = 0;
        if (isNaN(wTicks) || wTicks < 0) wTicks = 40;
        if (isNaN(mTry) || mTry < 1) mTry = 5;
        if (isNaN(gCD) || gCD < 0) gCD = 180;
        
        config.tpr.dimsEnabled = [e0, e1, e2];
        config.tpr.costPerDim = [c0, c1, c2];
        config.tpr.maxRangePerDim = [r0, r1, r2];
        config.tpr.centerMethod = (centerMethodIdx === 1 ? "coord" : "player");
        config.tpr.centerCoord = [centerX, 64, centerZ]; 
        config.tpr.waitTicksForChunkLoad = wTicks;
        config.tpr.maxTries = mTry;
        config.tpr.globalCooldownSec = gCD;
        
        saveConfig();
        
        pl.tell("§a已更新随机传送配置");
    });
}

//---------------------------------------------------------------TPZ 

function showTpzMenu(player) {
    let form = mc.newCustomForm();
    form.setTitle("§a自定义传送");
    form.addInput("§e请输入坐标 (格式: x y z )", "100 64 100", "");
    
    let currentDim = player.pos.dimid;
    let dimLabels = [0, 1, 2].map((idx) => {
        let cost = config.tpz.dimsCost[idx] || 0;
        let limit = config.tpz.dimsRadius[idx] || 999999;
        let dimText = `${getDimName(idx)} (价格:${cost}, 半径:${limit})`;
        
        if (config.tpz.dimsEnabled[idx] === false) {
            dimText += " §c---未启用";
        }
        
        return dimText;
    });
    
    let defIndex = Math.min(Math.max(0, currentDim), 2);
    form.addDropdown("§e选择维度", dimLabels, defIndex);
    
    if (player.isOP()) {
        form.addSwitch("§c自定义传送管理(打开后进入管理页面)", false);
    }
    
    player.sendForm(form, (pl, data) => {
        if (!data) {
            pl.tell("§c操作已取消");
            return;
        }
        
        let idx = 0;
        let inputCoord = data[idx++];
        let dimSelected = data[idx++];
        let manage = false;
        
        if (pl.isOP()) {
            manage = data[idx++];
        }
        
        if (manage === true) {
            showTpzConfigForm(pl);
            return;
        }
        
        let coords = parseCoordinates(inputCoord);
        if (!coords) {
            pl.tell("§c坐标格式错误，请使用 'x y z'");
            return;
        }
        
        let [xx, yy, zz] = coords;
        let dimId = dimSelected;
        
        if (!config.tpz.dimsEnabled[dimId]) {
            pl.tell(`§c该维度已被禁用，无法传送。`);
            return;
        }
        
        let cost = config.tpz.dimsCost[dimId] || 0;
        let limit = config.tpz.dimsRadius[dimId] || 999999;
        let dist = calcDistance(pl.pos.x, pl.pos.y, pl.pos.z, xx, yy, zz);
        
        if (dist > limit) {
            pl.tell(`§c超出最大传送范围(${limit}),无法传送.`);
            return;
        }
        
        if (cost > 0) {
            let xuid = pl.xuid;
            let bal = getBalance(xuid);
            
            if (bal < cost) {
                pl.tell(`§c余额不足! 需要${cost}, 当前:${bal || 0}`);
                return;
            }
            
            let ok = reduceMoney(xuid, cost);
            if (!ok) {
                pl.tell(`§c扣费失败,无法传送!`);
                return;
            }
        }
        
        let newPos = new FloatPos(xx, yy, zz, dimId);
        let ok = pl.teleport(newPos);
        
        if (ok) {
            pl.tell(`§a已传送到 [${xx},${yy},${zz}] (${getDimName(dimId)})，消耗:${cost}`);
            playTeleportSound(pl, "tpz");  
        } else {
            pl.tell(`§c传送失败(坐标或维度问题)!`);
        }
    });
}

function showTpzConfigForm(player) {
    let form = mc.newCustomForm();
    form.setTitle("§c自定义传送 - 管理界面");
    
    form.addSwitch(`§a启用主世界传送`, config.tpz.dimsEnabled[0]);
    form.addSwitch(`§c启用地狱传送`, config.tpz.dimsEnabled[1]);
    form.addSwitch(`§d启用末地传送`, config.tpz.dimsEnabled[2]);
    
    form.addInput(`§a主世界传送价格`, "", String(config.tpz.dimsCost[0] || 0));
    form.addInput(`§c地狱传送价格`, "", String(config.tpz.dimsCost[1] || 0));
    form.addInput(`§d末地传送价格`, "", String(config.tpz.dimsCost[2] || 0));
    
    form.addInput(`§a主世界最大半径`, "", String(config.tpz.dimsRadius[0] || 999999));
    form.addInput(`§c地狱最大半径`, "", String(config.tpz.dimsRadius[1] || 999999));
    form.addInput(`§d末地最大半径`, "", String(config.tpz.dimsRadius[2] || 999999));
    
    player.sendForm(form, (pl, data) => {
        if (!data) {
            pl.tell("§c操作已取消.");
            return;
        }
        
        let idx = 0;
        let e0 = data[idx++];
        let e1 = data[idx++];
        let e2 = data[idx++];
        let c0 = parseInt(data[idx++]);
        let c1 = parseInt(data[idx++]);
        let c2 = parseInt(data[idx++]);
        let r0 = parseInt(data[idx++]);
        let r1 = parseInt(data[idx++]);
        let r2 = parseInt(data[idx++]);
        
        if (isNaN(c0) || c0 < 0) c0 = 0;
        if (isNaN(c1) || c1 < 0) c1 = 0;
        if (isNaN(c2) || c2 < 0) c2 = 0;
        if (isNaN(r0) || r0 < 0) r0 = 999999;
        if (isNaN(r1) || r1 < 0) r1 = 999999;
        if (isNaN(r2) || r2 < 0) r2 = 999999;
        
        config.tpz.dimsEnabled = [e0, e1, e2];
        config.tpz.dimsCost = [c0, c1, c2];
        config.tpz.dimsRadius = [r0, r1, r2];
        
        saveConfig();
        
        pl.tell("§a配置已保存！");
    });
}

function parseCoordinates(coordStr) {
    if (!coordStr || typeof coordStr !== "string") return null;
    
    let parts = coordStr.trim().split(/\s+/);
    if (parts.length < 3) return null;
    
    let x = parseFloat(parts[0]), y = parseFloat(parts[1]), z = parseFloat(parts[2]);
    if (isNaN(x) || isNaN(y) || isNaN(z)) return null;
    
    return [x, y, z];
}

function calcDistance(x1, y1, z1, x2, y2, z2) {
    let dx = x2 - x1, dy = y2 - y1, dz = z2 - z1;
    return Math.sqrt(dx * dx + dy * dy + dz * dz);
}

//---------------------------------------------------------------WARP
const lastWarpUseMap = {};

function showWarpMenu(player) {
    let fm = mc.newSimpleForm();
    fm.setTitle("§a公共传送系统");
    
    let warps = Object.keys(warpsData);
    let isOp = player.isOP();
    
    let content = "§e请选择要传送的目标(若有):\n";
    if (warps.length === 0) {
        content += "§c当前还没有传送点.";
    }
    
    fm.setContent(content);
    
    let warpBtnData = [];
    warps.forEach(wName => {
        let wObj = warpsData[wName];
        let line1 = `§s${wName}`;
        let line2 = `§s冷却:${wObj.cdSec || 0}s | 花费:${wObj.cost || 0}`;
        fm.addButton(line1 + "\n" + line2);
        warpBtnData.push(wName);
    });
    
    let addWarpIndex = -1, configWarpIndex = -1;
    if (isOp) {
        addWarpIndex = warpBtnData.length;
        fm.addButton("§d添加传送点", "textures/ui/plus");
        configWarpIndex = warpBtnData.length + 1;
        fm.addButton("§c修改配置传送点", "textures/ui/anvil_icon");
    }
    
    player.sendForm(fm, (pl, id) => {
        if (id == null) return;
        
        if (id < warpBtnData.length) {
            let chosen = warpBtnData[id];
            doWarpTeleport(pl, chosen);
        } else if (isOp) {
            if (id === addWarpIndex) {
                addWarpForm(pl);
            } else if (id === configWarpIndex) {
                selectWarpToConfigForm(pl);
            }
        }
    });
}

function doWarpTeleport(player, warpName) {
    let wObj = warpsData[warpName];
    if (!wObj) {
        player.tell(`§c不存在传送点 [${warpName}]`);
        return;
    }
    
    let xuid = player.xuid || "NO_XUID";
    if (!lastWarpUseMap[warpName]) lastWarpUseMap[warpName] = {};
    
    let lastTime = lastWarpUseMap[warpName][xuid] || 0;
    let cdSec = wObj.cdSec || 0;
    let now = Date.now();
    let passed = Math.floor((now - lastTime) / 1000);
    let remain = cdSec - passed;
    
    if (remain > 0) {
        player.tell(`§c该传送点仍在冷却中,剩余${remain}秒`);
        return;
    }
    
    let cost = wObj.cost || 0;
    if (cost > 0) {
        let bal = getBalance(xuid);
        if (bal < cost) {
            player.tell(`§c余额不足,需要${cost},当前${bal}.`);
            return;
        }
        
        let ok = reduceMoney(xuid, cost);
        if (!ok) {
            player.tell(`§c扣费失败或余额不足.`);
            return;
        }
    }
    
    let exactPos = new FloatPos(
        parseFloat(wObj.x), 
        parseFloat(wObj.y), 
        parseFloat(wObj.z), 
        parseInt(wObj.dim) || 0
    );
    let success = player.teleport(exactPos);
    
    if (success) {
        player.tell(`§a已传送到 [${warpName}] cost:${cost}`);
        playTeleportSound(player, "warp");
        lastWarpUseMap[warpName][xuid] = now;
    } else {
        player.tell(`§c传送失败(坐标/维度无效).`);
    }
}

function addWarpForm(player) {
    let px = Math.floor(player.pos.x), py = Math.floor(player.pos.y), pz = Math.floor(player.pos.z);
    let dimid = player.pos.dimid;
    
    let fm = mc.newCustomForm();
    fm.setTitle("§d添加传送点");
    fm.addLabel(`§a若坐标留空则默认当前坐标:\n(你当前: x=${px},y=${py},z=${pz},dim=${getDimName(dimid)})`);
    fm.addInput("§e坐标(空格分割)", "", `${px} ${py} ${pz}`);
    fm.addInput("§c维度(0/1/2)", "", String(dimid));
    fm.addInput("§b传送点名字(不超过12字符)", "Warp1", "");
    fm.addInput("§a冷却时间(秒)", "", "0");
    fm.addInput("§s传送花费", "", "0");
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let coordStr = data[1].trim();
        let dimStr = data[2].trim();
        let wName = data[3].trim();
        let cdSec = parseInt(data[4]);
        let cost = parseInt(data[5]);
        
        if (!wName || wName.length > 12) {
            pl.tell("§c传送点名不合法.");
            return;
        }
        
        if (isNaN(cdSec) || cdSec < 0) cdSec = 0;
        if (isNaN(cost) || cost < 0) cost = 0;
        
        let dd = parseInt(dimStr);
        if (isNaN(dd) || dd < 0 || dd > 2) dd = 0;
        
        let x = px, y = py, z = pz;
        if (coordStr) {
            let parts = coordStr.split(/\s+/);
            if (parts.length >= 3) {
                let xx = parseFloat(parts[0]), yy = parseFloat(parts[1]), zz = parseFloat(parts[2]);
                if (!isNaN(xx)) x = xx;
                if (!isNaN(yy)) y = yy;
                if (!isNaN(zz)) z = zz;
            }
        }
        
        if (warpsData[wName]) {
            pl.tell(`§c已存在同名传送点[${wName}]，请换个名字或修改配置.`);
            return;
        }
        
        warpsData[wName] = {
            x: x, y: y, z: z, dim: dd,
            cdSec: cdSec,
            cost: cost
        };
        
        saveWarpsData();
        
        pl.tell(`§a成功添加传送点[${wName}] (dim:${dd},cd:${cdSec},cost:${cost}).`);
    });
}

function selectWarpToConfigForm(player) {
    let names = Object.keys(warpsData);
    if (names.length === 0) {
        player.tell("§c目前无传送点可配置.");
        return;
    }
    
    let fm = mc.newCustomForm();
    fm.setTitle("§c选择传送点进行配置");
    fm.addDropdown("选择传送点:", names, 0);
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let selIdx = data[0];
        if (selIdx < 0 || selIdx >= names.length) {
            pl.tell("§c选择无效");
            return;
        }
        
        let wName = names[selIdx];
        editWarpForm(pl, wName);
    });
}

function editWarpForm(player, warpName) {
    let wObj = warpsData[warpName];
    if (!wObj) {
        player.tell(`§c传送点[${warpName}]不存在.`);
        return;
    }
    
    let fm = mc.newCustomForm();
    fm.setTitle(`§c配置 [${warpName}]`);
    fm.addLabel("§a坐标留空则不变.维度(0/1/2).\n开启删除开关后点提交=删除该传送点.\n如需重命名，在下方输入新名称(不能与已有重名).");
    
    let coordHint = `${wObj.x} ${wObj.y} ${wObj.z}`;
    fm.addInput("§e坐标(x y z)", coordHint, "");
    fm.addInput("§c维度", "", String(wObj.dim || 0));
    fm.addInput("§b新名字(留空则不改)", "", "");
    fm.addInput("§s冷却(秒)", "", String(wObj.cdSec || 0));
    fm.addInput("§e花费", "", String(wObj.cost || 0));
    fm.addSwitch("§c删除此传送点?", false);
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let coordsStr = data[1].trim();
        let dimStr = data[2].trim();
        let newName = data[3].trim();
        let newCdSec = parseInt(data[4]);
        let newCost = parseInt(data[5]);
        let delSw = data[6];
        
        if (delSw === true) {
            delete warpsData[warpName];
            saveWarpsData();
            pl.tell(`§a已删除传送点[${warpName}]`);
            return;
        }
        
        if (isNaN(newCdSec) || newCdSec < 0) newCdSec = 0;
        if (isNaN(newCost) || newCost < 0) newCost = 0;
        
        let dd = parseInt(dimStr);
        if (isNaN(dd) || dd < 0 || dd > 2) dd = wObj.dim || 0;
        
        let x = wObj.x, y = wObj.y, z = wObj.z;
        if (coordsStr) {
            let parts = coordsStr.split(/\s+/);
            if (parts.length >= 3) {
                let xx = parseFloat(parts[0]), yy = parseFloat(parts[1]), zz = parseFloat(parts[2]);
                if (!isNaN(xx)) x = xx;
                if (!isNaN(yy)) y = yy;
                if (!isNaN(zz)) z = zz;
            }
        }
        
        let finalName = warpName;
        if (newName) {
            if (warpsData[newName] && newName !== warpName) {
                pl.tell(`§c已存在同名传送点[${newName}],无法重命名.`);
                return;
            }
            finalName = newName;
        }
        
        if (finalName !== warpName) {
            delete warpsData[warpName];
        }
        
        warpsData[finalName] = {
            x: x, y: y, z: z, dim: dd,
            cdSec: newCdSec,
            cost: newCost
        };
        
        saveWarpsData();
        
        pl.tell(`§a已更新传送点[${finalName}] (dim:${dd},cd:${newCdSec},cost:${newCost}).`);
    });
}

//---------------------------------------------------------------CONFIG
function showConfigMenu(player) {
    let fm = mc.newSimpleForm();
    fm.setTitle("§b系统配置");
    fm.setContent("§e请选择要配置的项目:");
    
    fm.addButton("§a全局经济设置", "textures/ui/icon_book_writable");
    fm.addButton("§d模块开关设置", "textures/ui/toggle_on");
    fm.addButton("§a家系统配置", "textures/ui/icon_recipe_nature");
    fm.addButton("§d互传系统配置", "textures/ui/dressing_room_skins");
    fm.addButton("§b随机传送配置", "textures/ui/icon_random");
    fm.addButton("§c坐标传送配置", "textures/ui/icon_map");
    fm.addButton("§e公共传送点配置", "textures/ui/icon_multiplayer");
    fm.addButton("§b跨服传送设置", "textures/ui/realms_key_art.png");
    fm.addButton("§c死亡点传送设置", "textures/ui/icon_alex");
    
    player.sendForm(fm, (pl, id) => {
        if (id === null) return;
        
        switch (id) {
            case 0:
                showEconomyConfigForm(pl);
                break;
            case 1:
                showModulesConfigForm(pl);
                break;
            case 2:
                showHomeConfigForm(pl);
                break;
            case 3:
                showTpaManageForm(pl);
                break;
            case 4:
                showTprConfigForm(pl);
                break;
            case 5:
                showTpzConfigForm(pl);
                break;
            case 6:
                selectWarpToConfigForm(pl);  
                break;
            case 7:
                showServerConfigForm(pl);
                break;
            case 8:
                showDeathTpConfigForm(pl);
                break;
        }
    });
}

function showEconomyConfigForm(player) {
    let fm = mc.newCustomForm();
    fm.setTitle("§a全局经济设置");
    
    let ecoList = ["llmoney", "scoreboard"];
    let defEco = (config.economyType === "scoreboard" ? 1 : 0);
    
    fm.addDropdown("§a经济体系", ecoList, defEco);
    fm.addInput("§e计分板名称(仅当scoreboard有效)", "money", config.scoreboardObj || "money");
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let ecoIdx = data[0];
        let sbName = data[1];
        
        let newEco = (ecoIdx === 1 ? "scoreboard" : "llmoney");
        
        config.economyType = newEco;
        config.scoreboardObj = sbName || "money";
        
        saveConfig();
        
        pl.tell(`§a已更新全局经济设置: 类型=${newEco}, 计分板=${sbName}`);
    });
}

function showModulesConfigForm(player) {
    let fm = mc.newCustomForm();
    fm.setTitle("§d模块开关设置");
    
    fm.addSwitch("§a启用家系统模块", config.modulesEnabled.home !== false);
    fm.addSwitch("§d启用互传系统模块", config.modulesEnabled.tpa !== false);
    fm.addSwitch("§b启用随机传送模块", config.modulesEnabled.tpr !== false);
    fm.addSwitch("§c启用坐标传送模块", config.modulesEnabled.tpz !== false);
    fm.addSwitch("§e启用公共传送点模块", config.modulesEnabled.warp !== false);
    fm.addSwitch("§b启用跨服传送模块", config.modulesEnabled.server !== false);
    fm.addSwitch("§c启用死亡点传送模块", config.modulesEnabled.deathtp !== false);
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        config.modulesEnabled = {
            home: data[0],
            tpa: data[1],
            tpr: data[2],
            tpz: data[3],
            warp: data[4],
            server: data[5],
            deathtp: data[6]
        };
        
        saveConfig();
        
        pl.tell("§a已更新模块开关设置!");
    });
}


//--------------------------------------------------------------- SYSTEM IMPLEMENTATION
let serversData = {};
if (!File.exists(SERVERS_PATH)) {
    serversData = {
        "生存服务器": {
            ip: "127.0.0.1",
            port: 19132,
            description: "主生存服务器"
        },
        "创造服务器": {
            ip: "127.0.0.1",
            port: 19133,
            description: "创造模式服务器"
        }
    };
    File.writeTo(SERVERS_PATH, JSON.stringify(serversData, null, 4));
} else {
    try {
        let raw = File.readFrom(SERVERS_PATH);
        serversData = JSON.parse(raw);
    } catch (e) {
        logger.error(`[${PLUGIN_NAME}] 读取服务器列表失败: ${e}`);
        serversData = {};
    }
}

function saveServersData() {
    if (!File.exists(BASE_DIR)) {
        File.createDir(BASE_DIR);
    }
    File.writeTo(SERVERS_PATH, JSON.stringify(serversData, null, 4));
}

function showServerMenu(player) {
    let fm = mc.newSimpleForm();
    fm.setTitle("§b跨服传送系统");
    fm.setContent("§e选择要传送到的服务器:");
    
    let servers = Object.keys(serversData);
    let serverBtnData = [];
    
    servers.forEach(serverName => {
        let server = serversData[serverName];
        let line1 = `§a${serverName}`;
        let line2 = `§c${server.description || ""} §f(${server.ip}:${server.port})`;
        fm.addButton(line1 + "\n" + line2);
        serverBtnData.push(serverName);
    });
    
    let configIdx = -1;
    
    if (player.isOP()) {
        configIdx = serverBtnData.length;
        fm.addButton("§c管理服务器", "textures/ui/settings_glyph_color_2x");
    }
    
    player.sendForm(fm, (pl, id) => {
        if (id === null) return;
        
        if (id < serverBtnData.length) {
            let serverName = serverBtnData[id];
            let server = serversData[serverName];
            
            if (!server) {
                pl.tell("§c服务器信息获取失败");
                return;
            }
            
            let confirmForm = mc.newSimpleForm();
            confirmForm.setTitle("§b确认跨服传送");
            confirmForm.setContent(`§e确定要传送到 §a${serverName} §e吗?\n§c${server.description || ""}\n§f${server.ip}:${server.port}`);
            confirmForm.addButton("§a确认传送");
            confirmForm.addButton("§c取消");
            
            pl.sendForm(confirmForm, (player, btnId) => {
                if (btnId === 0) {
                    player.tell(`§a正在将你传送到 ${serverName}...`);
                    let success = player.transServer(server.ip, server.port);
                    
                    if (!success) {
                        player.tell("§c传送失败，请检查服务器地址是否正确");
                    }
                }
            });
        } else if (id === configIdx) {
            showServerConfigMenu(pl);
        }
    });
}

function showServerConfigMenu(player) {
    let fm = mc.newSimpleForm();
    fm.setTitle("§c服务器管理");
    fm.setContent("§e选择操作:");
    
    fm.addButton("§a添加服务器", "textures/ui/plus");
    fm.addButton("§c删除服务器", "textures/ui/trash");
    fm.addButton("§b修改服务器", "textures/ui/pencil_edit_icon");
    fm.addButton("§d修改跨服设置", "textures/ui/settings_glyph_color_2x");
    
    player.sendForm(fm, (pl, id) => {
        if (id === null) return;
        
        switch (id) {
            case 0:
                showAddServerForm(pl);
                break;
            case 1:
                showDeleteServerForm(pl);
                break;
            case 2:
                showSelectServerToEditForm(pl);
                break;
            case 3:
                showServerConfigForm(pl);
                break;
        }
    });
}

function showAddServerForm(player) {
    let fm = mc.newCustomForm();
    fm.setTitle("§a添加服务器");
    
    fm.addInput("§e服务器名称", "生存服务器", "");
    fm.addInput("§aIP地址", "127.0.0.1", "");
    fm.addInput("§b端口", "19132", "");
    fm.addInput("§d描述(可选)", "这是一个服务器", "");
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let serverName = (data[0] || "").trim();
        let ip = (data[1] || "").trim();
        let portStr = (data[2] || "").trim();
        let description = data[3] || "";
        
        if (!serverName) {
            pl.tell("§c服务器名称不能为空");
            return;
        }
        
        if (!ip) {
            pl.tell("§cIP地址不能为空");
            return;
        }
        
        if (!portStr) {
            pl.tell("§c端口不能为空");
            return;
        }
        
        let port = parseInt(portStr);
        if (isNaN(port) || port < 1 || port > 65535) {
            pl.tell("§c端口必须是1-65535之间的数字");
            return;
        }
        
        if (serversData[serverName]) {
            pl.tell("§c已存在同名服务器，请使用不同的名称");
            return;
        }
        
        serversData[serverName] = {
            ip: ip,
            port: port,
            description: description
        };
        
        saveServersData();
        pl.tell(`§a已添加服务器 ${serverName}`);
    });
}

function showDeleteServerForm(player) {
    let serverNames = Object.keys(serversData);
    
    if (serverNames.length === 0) {
        player.tell("§c没有可删除的服务器");
        return;
    }
    
    let fm = mc.newCustomForm();
    fm.setTitle("§c删除服务器");
    fm.addDropdown("§e选择要删除的服务器", serverNames, 0);
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let selectedIdx = data[0];
        if (selectedIdx < 0 || selectedIdx >= serverNames.length) {
            pl.tell("§c选择无效");
            return;
        }
        
        let serverName = serverNames[selectedIdx];
        
        let confirmForm = mc.newSimpleForm();
        confirmForm.setTitle("§c确认删除");
        confirmForm.setContent(`§e确定要删除服务器 §a${serverName} §e吗?`);
        confirmForm.addButton("§c确认删除");
        confirmForm.addButton("§a取消");
        
        pl.sendForm(confirmForm, (player, btnId) => {
            if (btnId === 0) {
                delete serversData[serverName];
                saveServersData();
                player.tell(`§a已删除服务器 ${serverName}`);
            }
        });
    });
}

function showSelectServerToEditForm(player) {
    let serverNames = Object.keys(serversData);
    
    if (serverNames.length === 0) {
        player.tell("§c没有可编辑的服务器");
        return;
    }
    
    let fm = mc.newCustomForm();
    fm.setTitle("§b编辑服务器");
    fm.addDropdown("§e选择要编辑的服务器", serverNames, 0);
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let selectedIdx = data[0];
        if (selectedIdx < 0 || selectedIdx >= serverNames.length) {
            pl.tell("§c选择无效");
            return;
        }
        
        let serverName = serverNames[selectedIdx];
        showEditServerForm(pl, serverName);
    });
}

function showEditServerForm(player, serverName) {
    let server = serversData[serverName];
    if (!server) {
        player.tell(`§c服务器 ${serverName} 不存在`);
        return;
    }
    
    let fm = mc.newCustomForm();
    fm.setTitle(`§b编辑 ${serverName}`);
    
    fm.addInput("§e新服务器名称(留空则不变)", serverName, "");
    fm.addInput("§aIP地址", server.ip, server.ip);
    fm.addInput("§b端口", String(server.port), String(server.port));
    fm.addInput("§d描述(可选)", server.description || "", server.description || "");
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let newName = (data[0] || "").trim();
        let ip = (data[1] || "").trim();
        let portStr = (data[2] || "").trim();
        let description = data[3] || "";
        
        if (newName && newName !== serverName && serversData[newName]) {
            pl.tell("§c已存在同名服务器，请使用不同的名称");
            return;
        }
        
        if (!ip) {
            pl.tell("§cIP地址不能为空");
            return;
        }
        
        if (!portStr) {
            pl.tell("§c端口不能为空");
            return;
        }
        
        let port = parseInt(portStr);
        if (isNaN(port) || port < 1 || port > 65535) {
            pl.tell("§c端口必须是1-65535之间的数字");
            return;
        }
        
        if (newName && newName !== serverName) {
            serversData[newName] = {
                ip: ip,
                port: port,
                description: description
            };
            delete serversData[serverName];
        } else {
            serversData[serverName] = {
                ip: ip,
                port: port,
                description: description
            };
        }
        
        saveServersData();
        pl.tell(`§a已更新服务器信息`);
    });
}

function showServerConfigForm(player) {
    let fm = mc.newCustomForm();
    fm.setTitle("§d跨服传送设置");
    
    fm.addSwitch("§a启用跨服传送功能", config.server.enabled !== false);
    
    let permOptions = ["所有玩家", "仅OP"];
    let permIdx = config.server.permissionLevel > 0 ? 1 : 0;
    fm.addDropdown("§e使用权限", permOptions, permIdx);
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        config.server.enabled = data[0];
        config.server.permissionLevel = data[1];
        
        saveConfig();
        pl.tell("§a已更新跨服传送设置");
    });
}

//--------------------------------------------------------------- DEATH

let deathRecordsData = {};
if (!File.exists(DEATH_RECORDS_PATH)) {
    File.writeTo(DEATH_RECORDS_PATH, JSON.stringify({}, null, 4));
} else {
    try {
        let raw = File.readFrom(DEATH_RECORDS_PATH);
        deathRecordsData = JSON.parse(raw);
    } catch (e) {
        logger.error(`[${PLUGIN_NAME}] 读取死亡记录失败: ${e}`);
        deathRecordsData = {};
    }
}

const lastDeathTpUse = {};

function saveDeathRecordsData() {
    if (!File.exists(BASE_DIR)) {
        File.createDir(BASE_DIR);
    }
    File.writeTo(DEATH_RECORDS_PATH, JSON.stringify(deathRecordsData, null, 4));
}

mc.listen("onPlayerDie", (player) => {
    if (!config.modulesEnabled.deathtp) {
        config.modulesEnabled.deathtp = true;
        saveConfig();
    }
    
    if (!config.deathtp) {
        config.deathtp = { ...defaultConfig.deathtp };
        saveConfig();
    }
    
    if (!config.modulesEnabled.deathtp || !config.deathtp.enabled) return;
    
    let xuid = player.xuid;
    let pos = player.pos;
    let now = Date.now();
    
    let deathRecord = {
        x: pos.x,
        y: pos.y,
        z: pos.z,
        dim: pos.dimid,
        time: now
    };
    
    let records = deathRecordsData[xuid] || [];
    records.unshift(deathRecord);
    
    if (records.length > config.deathtp.maxRecords) {
        records = records.slice(0, config.deathtp.maxRecords);
    }
    
    deathRecordsData[xuid] = records;
    saveDeathRecordsData();
    
    player.tell("§c你的死亡位置已记录，复活后可使用 /tpdeath 传送回死亡点");
});

function showDeathTpMenu(player) {
    let xuid = player.xuid;
    let records = deathRecordsData[xuid] || [];
    
    if (records.length === 0) {
        player.tell("§c你没有死亡记录");
        return;
    }
    
    if (!config.deathtp) {
        config.deathtp = { ...defaultConfig.deathtp };
        saveConfig();
    }
    
    let fm = mc.newSimpleForm();
    fm.setTitle("§c死亡点传送");
    
    let invincibleText = config.deathtp.invincibleEnabled ? 
        `§e传送后将获得 §a${config.deathtp.invincibleDuration || 5}秒 §e无敌时间保护\n` : "";
    
    fm.setContent(`§e选择要传送到的死亡点:
${invincibleText}§c记录最近的 ${config.deathtp.maxRecords} 次死亡位置`);
    
    let baseCost = config.deathtp.baseCost || 100;
    let multiplier = config.deathtp.costMultiplier || 1.5;
    
    records.forEach((record, index) => {
        let cost = Math.floor(baseCost * Math.pow(multiplier, index));
        let time = new Date(record.time).toLocaleString();
        let dim = getDimName(record.dim);
        
        let line1 = `§c死亡 #${index + 1} §f(${Math.floor(record.x)}, ${Math.floor(record.y)}, ${Math.floor(record.z)})`;
        let line2 = `§b${dim} §f| §e费用: ${cost} §f| §c${time}`;
        
        fm.addButton(line1 + "\n" + line2);
    });
    
    if (player.isOP()) {
        fm.addButton("§c管理死亡点系统", "textures/ui/settings_glyph_color_2x");
    }
    
    player.sendForm(fm, (pl, id) => {
        if (id === null) return;
        
        if (id < records.length) {
            teleportToDeathPoint(pl, id);
        } else if (pl.isOP()) {
            showDeathTpConfigForm(pl);
        }
    });
}

function teleportToDeathPoint(player, recordIndex) {
    let xuid = player.xuid;
    let records = deathRecordsData[xuid] || [];
    
    if (recordIndex < 0 || recordIndex >= records.length) {
        player.tell("§c无效的死亡记录索引");
        return;
    }
    
    if (!config.deathtp) {
        config.deathtp = { ...defaultConfig.deathtp };
        saveConfig();
    }
    
    let record = records[recordIndex];
    let baseCost = config.deathtp.baseCost || 100;
    let multiplier = config.deathtp.costMultiplier || 1.5;
    let cost = Math.floor(baseCost * Math.pow(multiplier, recordIndex));
    
    let now = Date.now();
    let lastUseTime = lastDeathTpUse[xuid] || 0;
    let cdSec = config.deathtp.cooldownSec || 60;
    let passed = Math.floor((now - lastUseTime) / 1000);
    let remain = cdSec - passed;
    
    if (remain > 0) {
        player.tell(`§c死亡点传送仍在冷却中，剩余${remain}秒`);
        return;
    }
    
    let bal = getBalance(xuid);
    if (bal < cost) {
        player.tell(`§c余额不足，需要${cost}，当前${bal}`);
        return;
    }
    
    let ok = reduceMoney(xuid, cost);
    if (!ok) {
        player.tell("§c扣费失败，可能余额不足");
        return;
    }
    
    let exactPos = new FloatPos(
        parseFloat(record.x),
        parseFloat(record.y),
        parseFloat(record.z),
        parseInt(record.dim)
    );
    let success = player.teleport(exactPos);
    
    if (success) {
        player.tell(`§a已传送到死亡点 #${recordIndex + 1}，花费: ${cost}`);
        playTeleportSound(player, "deathtp");  
        
        lastDeathTpUse[xuid] = now; 
        
        if (config.deathtp.invincibleEnabled) {
            let duration = config.deathtp.invincibleDuration || 5;
            let level = config.deathtp.resistanceLevel || 4;
            
            player.addEffect(11, duration * 20, level, false);
            player.tell(`§a获得 ${duration} 秒无敌时间保护`);
        }
    } else {
        player.tell("§c传送失败，可能是坐标无效");
        addMoney(xuid, cost); 
    }
}

function showDeathTpConfigForm(player) {
    let fm = mc.newCustomForm();
    fm.setTitle("§c死亡点传送配置");
    
    if (!config.deathtp) {
        config.deathtp = { ...defaultConfig.deathtp };
        saveConfig();
    }
    
    fm.addSwitch("§a启用死亡点传送", config.deathtp.enabled !== false);
    fm.addInput("§e最大记录数量", "5", String(config.deathtp.maxRecords || 5));
    fm.addInput("§b基础传送费用", "100", String(config.deathtp.baseCost || 100));
    fm.addInput("§c费用倍率", "1.5", String(config.deathtp.costMultiplier || 1.5));
    fm.addInput("§d传送冷却(秒)", "60", String(config.deathtp.cooldownSec || 60));
    
    fm.addSwitch("§a启用传送后无敌时间", config.deathtp.invincibleEnabled !== false);
    fm.addInput("§e无敌时间持续秒数", "5", String(config.deathtp.invincibleDuration || 5));
    fm.addInput("§b抗性效果等级(0-50)", "4", String(config.deathtp.resistanceLevel || 4));
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let enabled = data[0];
        let maxRecords = parseInt(data[1]);
        let baseCost = parseInt(data[2]);
        let multiplier = parseFloat(data[3]);
        let cooldown = parseInt(data[4]);
        let invincibleEnabled = data[5];
        let invincibleDuration = parseInt(data[6]);
        let resistanceLevel = parseInt(data[7]);
        
        if (isNaN(maxRecords) || maxRecords < 1) maxRecords = 5;
        if (isNaN(baseCost) || baseCost < 0) baseCost = 100;
        if (isNaN(multiplier) || multiplier < 1) multiplier = 1.5;
        if (isNaN(cooldown) || cooldown < 0) cooldown = 60;
        if (isNaN(invincibleDuration) || invincibleDuration < 0) invincibleDuration = 5;
        if (isNaN(resistanceLevel) || resistanceLevel < 0 || resistanceLevel > 50) resistanceLevel = 4;
        
        config.deathtp = {
            enabled: enabled,
            maxRecords: maxRecords,
            baseCost: baseCost,
            costMultiplier: multiplier,
            cooldownSec: cooldown,
            invincibleEnabled: invincibleEnabled,
            invincibleDuration: invincibleDuration,
            resistanceLevel: resistanceLevel
        };
        
        saveConfig();
        pl.tell("§a死亡点传送配置已更新");
    });
}

function showClearDeathRecordsForm(player) {
    let playerNames = [];
    let playerXuids = [];
    
    for (let xuid in deathRecordsData) {
        let p = mc.getPlayer(xuid);
        let name = p ? p.name : xuid; 
        playerNames.push(name);
        playerXuids.push(xuid);
    }
    
    if (playerNames.length === 0) {
        player.tell("§c没有任何死亡记录可清除");
        return;
    }
    
    let fm = mc.newCustomForm();
    fm.setTitle("§c清除死亡记录");
    fm.addDropdown("§e选择要清除记录的玩家", playerNames, 0);
    fm.addSwitch("§c清除所有记录", false);
    
    player.sendForm(fm, (pl, data) => {
        if (!data) return;
        
        let selectedIdx = data[0];
        let clearAll = data[1];
        
        if (clearAll) {
            deathRecordsData = {};
            saveDeathRecordsData();
            pl.tell("§a已清除所有玩家的死亡记录");
        } else {
            if (selectedIdx < 0 || selectedIdx >= playerXuids.length) {
                pl.tell("§c选择无效");
                return;
            }
            
            let xuid = playerXuids[selectedIdx];
            let name = playerNames[selectedIdx];
            
            delete deathRecordsData[xuid];
            saveDeathRecordsData();
            pl.tell(`§a已清除玩家 ${name} 的死亡记录`);
        }
    });
}

//--------------------------------------------------------------设置播放音效

function playTeleportSound(player, moduleType, delayMs = 500) {
    let sound = "entity.enderman.teleport"; 
    
    switch(moduleType) {
        case "home":
            sound = "block.portal.travel"; //传送门音效
            break;
        case "tpa":
            sound = "entity.enderman.teleport"; //末影人传送音效
            break;
        case "tpr":
            sound = "mob.shulker.teleport"; //潜影贝传送音效
            break;
        case "tpz":
            sound = "entity.ender_pearl.throw"; //末影珍珠音效
            break;
        case "warp":
            sound = "beacon.activate"; //信标激活音效
            break;
        case "deathtp":
            sound = "mob.wither.spawn"; //凋灵生成音效
            break;
    }
    
    setTimeout(() => {
        mc.runcmdEx(`playsound ${sound} @a[name="${player.name}"]`);
    }, delayMs);
}

logger.info(`[${PLUGIN_NAME}] 已加载, 版本: ${PLUGIN_VERSION.join('.')}`);